// Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef SHAREDDATA_HPP
#define SHAREDDATA_HPP
#include <gtkmm.h>

#include <list>
#include <stack>
#include <string>
#include "ourmove.hpp"
#include "field.hpp"
#include "config.h"


/** Used to categorize the requested moves.
 */

typedef struct {
  /** Is there really a requested move.
      This option is used by check_moves().
   */
  bool requested;
  /** Player's color
   */
  bool color;
  /** Set if the previous move of this color was wrong.
   */
  move_signal sig;
} request_t;

/** Communication between game logic an gui.
    Via this class the game logic and the gui, both running in different threads, can exchange moves
 */

class shareddata{
public:
  shareddata();
  
  /** Indicate that a certain move is requested.
      A move request is placed in a message queue.
      This method is normally called by a human player who wants to get a move from the gui.
      @param color Player's color
      @param sig If true the previous move of the same color was not allowed
      @return Returns the move which was requested after waiting for its production
   */
  ourmove_t request_move(bool color, move_signal sig);

  /** Checks if there is a request for a move.
      Called by the gui to get to know if input is needed.
      @return Characteristics of the requested move. If checkmoves().requested == 0 no move has been requested
   */
  request_t check_moves();

  /** Used by the gui to create a new move that is received by a human player.
      Normally the gui calls this method as a reaction in the result of check_moves().
      @param request Characteristics of the move that has to be added. Normally the characteristics received from check_moves() fit here. 
      @param move The move to be added.
      @return Shows whether the addition of the move was successful.
   */
  bool add_move(request_t request, ourmove_t move);


  /**
     The observer uses this function to indicate fields whose configuration changed.
     This this information is inserted into a queue.
     @param move The precise configuration of the changed field
   */
  void add_move_to_display(fieldvec_t move);

  /** First entry of the queue is removed.
      Called by the gui to get the changes calculated by the game logic
      @return Removed entry.
   */
  fieldvec_t rem_move_to_display();

  /** Checks whether there is still information in the queue that can be displayed.
   */
  bool no_moves_to_display();

 
  /** Sends the logic a signal to kill itself
  */
  void kill_logic();
  
  /** Sends the go_back signal to remove the last move
  */
  void go_back(request_t re);
  
  /** Uses Mr Proper to clean all moves, requests and similar things
  		2 be used after deleting a logic
  */
  void clean_up();

  /** Sets the number of computer players 
  */
  void setnumberai(int number);

  /** Returns the number of computer players
   */
  int getnumberai();

  /** Dumps how many moves there are in the queues
   */
  void dump_moves_in_queue();

  /** Checks if the move history is empty.
   */
  bool history_empty();

  void reconfigure_players(int ai_players);

  bool save_history(std::string newfile); 
  bool open_history(std::string newfile); 
  
  void reset_player_request();


  std::list<ourmove_t> history;
  std::list<ourmove_t> load_history;
  
private:
  Glib::Mutex mut_requests;
  Glib::Cond move_available; 
  Glib::Mutex mut_moves_to_display;
  Glib::Cond everything_displayed;
  Glib::Mutex mut_numberai;
  
  std::list<request_t> move_requests;
  std::list<ourmove_t> moves_black;
  std::list<ourmove_t> moves_white;
  std::list<fieldvec_t> moves_to_display;

  int numberai;
  int numberai_backup;

  int load_history_flag;
  /** Sends the update all signal to the players
  */
  void update_all();
};

#endif //SHAREDDATA_HPP  
